/*****************************************************************
 * Project:         Software Reverse Engineering (SRE) Education
 *                  Malware, Adware, and Spyware
 *                  Alarm Clock Trojan (Malware) Application
 * Programmer:      Teodoro Cipresso, San Jose State University
 *                  tcipress@hotmail.com
 ******************************************************************
 */
package info.reversingproject.alarmclock;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * An instance of this class is used to store the set alarm date and time. A
 * convenience method is provided for accessing the current date and time of the
 * system.
 */
public class AlarmClockModel {

	private static final String _datePattern = "MM/dd/yy HH:mm:ss";
	private static SimpleDateFormat _dateFormat = new SimpleDateFormat(
			_datePattern);
	private Date alarmTime = null;

	/**
	 * @return the current set alarm time or null if not set
	 */
	public Date getAlarmTime() {
		return alarmTime;
	} // getAlarmTime

	/**
	 * @param alarmTime
	 *            , the alarm time set. It's assumed that the controller has
	 *            validated this alarm time.
	 */
	public synchronized void setAlarmTime(Date alarmTime) {
		this.alarmTime = alarmTime;
	} // setAlarmTime

	/**
	 * @return a Date object representing the current system time.
	 */
	public Date getClockTime() {
		return new Date(System.currentTimeMillis());
	} // getClockTime

	/**
	 * @return the date format used for the alarm and current date time.
	 */
	public static SimpleDateFormat getDateFormat() {
		return _dateFormat;
	} // getDateFormat

	/**
	 * @return the pattern used to parse and format the date and time.
	 */
	public static String getDatePattern() {
		return _datePattern;
	} // getDatePattern
} // AlarmClockModel
